#<profile4xi profile = "02" />
# ------------------------------------------------------------------------------------------------- 
#<program4xi program = "finalize4xi.py">
#<abstract4xi abstract = "provides Finalize4Xi class"/>
#<refer4xi reference = "util.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Finalize4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import itertools
    import weakref
    import atexit         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi 
from constant_managers4xi import get_shutdown_manager4xi    
from finalizer4xi import Finalizer4Xi

from print4xi import print4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#/>

# =================================================================================================
#<class4xi class = "Finalize4Xi" base = "object" child = "None" synopsis = "supports object finalization using weakrefs" usage = "
#     connection.py:
#         self._unlink = Finalize(debug('failed to connect to address %s', address))
#     forking.py:
#         self._terminate = Finalize(info('calling terminate() for daemon %s', p.name))
#     heap.py:
#         Finalize(self, BufferWrapper._heap.free, args=(block,))
#     managers.py:
#         self.shutdown = Finalize(util.info('manager serving at %r', server.address))
#     pool.py: 
#         from multiprocessingi import Finalize, debug
#          self._terminate = Finalizei(
#                                        debug('task handler found thread._state != RUN')
#                                        debug('could not put task on queue')
#                                        debug('doing set_length()')
#     queues.py:
#             self._jointhread = Finalize(
#                 self._thread, Queue._finalize_join,
#                 [weakref.ref(self._thread)],
#                 exitpriority=-5
#                 )
#
# basemanager4xi.py:
#     self.shutdown = Finalize4Xi(self, type(self)._finalize_manager,args=(self._process, self._address, self._authkey,self._state, self._Client),exitpriority=0)
#
# baseproxy4xi.py:
#     self._close = Finalize4Xi(self, BaseProxy4Xi._decref,args=(self._token, self._authkey, state,self._tls, self._idset, self._Client),exitpriority=get_exitpriority_manager4xi())
#
# bufferwrapper4xi.py:
#     Finalize4Xi(self, BufferWrapper4Xi._heap.free, args=(block2xi,)) 
#
# get_temp_dir4xi.py:
#     Finalize4Xi(None, shutil.rmtree, args=[tempdir2xi], exitpriority=-100)
#
# pool4xi.py:
#     self._terminate = Finalize4Xi(self, self._terminate_pool, args=(self._taskqueue, self._inqueue, self._outqueue, self._pool,self._task_handler, self._result_handler, self._cache),exitpriority= get_exitpriority_pool4xi())
#
# queue4xi.py:
#     self._close = Finalize4Xi(self._thread, Queue._finalize_join,[weakref.ref(self._thread)],exitpriority=-5)
#
# socketlistener4xi.py:
#     self._unlink = Finalize4Xi(self, os.unlink, args=(address,), exitpriority=0)
#
# syncmanager4xi.py:
#     self.shutdown = Finalize4Xi(self, type(self)._finalize_manager,args=(self._process, self._address, self._authkey,self._state, self._Client),exitpriority=0)"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class Finalize4Xi(object): 
    '''
    Finalize4Xi Class supports object finalization using weakrefs
    '''
    
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('still_active', 'cancel', 'sense', 'reason', 'ace')      
    #"
    #/>
    
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _registry = {}
    _counter = itertools.count()
    #"
    #/>
    
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Finalize4Xi" usage = "initializes Finalize4Xi">                    
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, obj, callback, args=(), kwargs=None, exitpriority=None):
        ''' 
        Finalize4Xi __init __ method initializes with (self, obj, callback, args, kwargs, exitpriority)  
        '''
        
        
        # -----------------------------------------------------------------------------------------
        # Step 1/5: OK
        # -----------------------------------------------------------------------------------------
        class2print = "Finalize4Xi" 
        method2print = "__init__"
        
        message2print = "{(Step 1/5a) started" + "; init-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:58:24 01/22/13 Pacific Standard Time
        # 18:40:44 02/22/13 Pacific Standard Time
        
        message2print = "{(Step 1/5b) passed" + "; object-passed-to-init-of-Finalize4Xi: " + str(obj) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <queue4xi.Queue4Xi object at 0x0000000003690550>
        # <bufferwrapper4xi.BufferWrapper4Xi object at 0x0000000002AC2D30>
         
        message2print = "{(Step 1/5c) passed" + "; callback-passed-to-init-of-Finalize4Xi: " + str(callback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # <function Queue4Xi._finalize_close at 0x0000000003667E18>
        # <bound method Heap4Xi.free of <heap4xi.Heap4Xi object at 0x0000000002AF16D8>>
         
        message2print = "{(Step 1/5d) passed" + "; args-passed-to-init-of-Finalize4Xi: " + str(args) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # [deque([]), <Condition(<_thread.lock object at 0x0000000003794238>, 0)>]
        # ((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096),)
             
        message2print = "{(Step 1/5e) passed" + "; kwargs-passed-to-init-of-Finalize4Xi: " + str(kwargs) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None
        # None
         
        message2print = "{(Step 1/5f) passed" + "; exitpriority-passed-to-init-of-Finalize4Xi: " + str(exitpriority) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 10
        # None
         
        message2print = "{(Step 1/5g) borning" + "; finalize2xi-of-Finalize4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 02:58:24 01/22/13 Pacific Standard Time
        # 18:40:44 02/22/13 Pacific Standard Time
        
        message2print = "{(Step 1/5h) inheriting" + "; object-inheriting-by-finalize2xi-of-Finalize4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>             
        
        message2print = "{(Step 1/5i) shared" + "; registry-shared-by-finalize2xi-of-Finalize4Xi: " + str(Finalize4Xi._registry)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # {}
        
        
        message2print = "{(Step 1/5j) shared" + "; counter-shared-by-finalize2xi-of-Finalize4Xi: " + str(Finalize4Xi._counter)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # count(0)
              
        # -----------------------------------------------------------------------------------------
        # Step 2/5: OK
        # -----------------------------------------------------------------------------------------
             
        assert exitpriority is None or type(exitpriority) is int

        message2print = "{(Step 2/5a) asserted" + "; exitpriority-asserted-by-init-of-Finalize4Xi: " + str(exitpriority) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # 10   
                      
        if obj is not None:
            self._weakref = weakref.ref(obj, self)
            
            message2print = "{(Step 2/5b) set" + "; weakref-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._weakref)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <weakref at 0x00000000036CE5E8; to 'Queue4Xi' at 0x0000000003690550>
            # <weakref at 0x00000000035FD6D8; to 'BufferWrapper4Xi' at 0x0000000002AC2D30>
        else:
            assert exitpriority is not None
            
            message2print = "{(Step 2/5c) asserted" + "; exitpriority-asserted-by-init-of-Finalize4Xi: " + str(exitpriority) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0
                            
        # -----------------------------------------------------------------------------------------
        # Step 3/5: OK
        # -----------------------------------------------------------------------------------------
           
        self._callback = callback
         
        message2print = "{(Step 3/5a) set" + "; callback-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <function Queue4Xi._finalize_close at 0x0000000003667E18>          
        # <bound method Heap4Xi.free of <heap4xi.Heap4Xi object at 0x0000000002AF16D8>>
                       
        self._args = args
          
        message2print = "{(Step 3/5b) set" + "; args-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._args)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # [deque([]), <Condition(<_thread.lock object at 0x0000000003794238>, 0)>]
        # ((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096),)               
                        
        self._kwargs = kwargs or {}
 
        message2print = "{(Step 3/5c) set" + "; kwargs-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._kwargs)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # {}
                      
  
        
        # -----------------------------------------------------------------------------------------
        # Step 4/5:
        # -----------------------------------------------------------------------------------------
        
        self._key = (exitpriority, next(Finalize4Xi._counter))  
 
        message2print = "{(Step 4/5a) set" + "; key-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._key)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # (10, 0)
        # (None, 0)
        
        self._pid = os.getpid()
        message2print = "{(Step 4/5b) set" + "; pid-of-finalize2xi-set-by-init-of-Finalize4Xi to: " + str(self._pid)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 3616
        # 2332
                          
        # -----------------------------------------------------------------------------------------
        # Step 5/5: 
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 5/5a) passing" + "; key-of-finalize2xi-passing-to-set-finalizer-registry-element-by-init-of-Finalize4Xi: " + str(self._key) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # (10, 0)
        # (None, 0)
        
        message2print = "{(Step 5/5b) used" + "; finalizer-registry-used-by-init-of-Finalize4Xi: " + str(Finalize4Xi._registry)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # {}
   
        
        try:
            Finalize4Xi._registry[self._key] = self          
               
         
            message2print = "{(Step 5/5c) set" + "; finalizer-registry-element-set-by-init-of-Finalize4Xi to: " + str(Finalize4Xi._registry[self._key])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <finalize4xi: __repr__: Finalize object, callback=free, args=((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096),)>
               
        except:
            message2print = "{(Step 5/5d) hit" + "; key-of-finalize2xi-hit-by-call-of-f2xi-Finalize4Xi: " + " *** " + str(self._key)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
                  
 
        message2print = "{(Step 5/5e) activated" + "; finalize2xi-of-Finalize4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 18:40:44 02/22/13 Pacific Standard Time
        
        message2print = "{(Step 5/5f) inherited" + "; object-inherited-by-finalize2xi-of-Finalize4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>                         
        
        message2print = "{(Step 5/5g) returning" + "; init-of-Finalize4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # 18:40:44 02/22/13 Pacific Standard Time                
                
                                        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "call-of-Finalize4Xi" usage = "runs the callback unless it has already been called or cancelled">                     
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __call__(self, wr=None):
        '''
        Finalize4Xi __call__ method runs the callback unless it has already been called or cancelled
        '''

        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "Finalize4Xi"
        method2print = "__call__"

        message2print = "{started" + "; call-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # 02:58:24 01/22/13 Pacific Standard Time
        
        message2print = "{passed" + "; wr-passed-to-call-of-Finalize4Xi: " + str(wr) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # <weakref at 0x00000000036CE5E8; dead>
        try:
            
            message2print = "{deleting" + "; key-of-finalize2xi-deleting-by-call-of-Finalize4Xi: " + str(self._key) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            # (10, 0)
                           
            del Finalize4Xi._registry[self._key]
            

                   
        except KeyError:
            
            message2print = "{hit" + "; key-of-finalize2xi-hit-by-call-of-Finalize4Xi: " + " *** " + str(self._key)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            raise        

        else:

            res = self._callback(*self._args, **self._kwargs)
            self._weakref = self._callback = self._args = \
                            self._kwargs = self._key = None
            
            message2print = "{returning" + "; res-returning-by-call-of-Finalize4Xi: " + str(res) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
            # None                  

            return res
    #</methodize4xi>
    
                
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "cancel-of-Finalize4Xi" usage = "finalizes an object">                  
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def cancel(self):
        '''
        Finalize4Xi cancel method finalizes an object
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "Finalize4Xi"
        method2print = "cancel"

        message2print = "{started" + "; cancel-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
 
        try:
            message2print = "{deleting" + "; key-of-finalize2xi-deleting-by-cancel-of-Finalize4Xi: " + str(self._key) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
            del Finalize4Xi._registryement[self._key]
            

                

        except KeyError:
            
            message2print = "{hit" + "; key-of-finalize2xi-hit-by-cancel-of-Finalize4Xi: " + " *** " + str(self._key)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
            raise
            
        else:
            self._weakref = self._callback = self._args = \
                            self._kwargs = self._key = None
                            
            message2print = "{set" + "; weakref-of-finalize2xi-set-by-cancel-of-Finalize4Xi to: " + str(self._weakref)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                
    #</methodize4xi>
                                            

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "still-active-of-Finalize4Xi" usage = "returns whether this finalizer is still waiting to invoke callback">                   
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def still_active(self):
        '''
        Finalize4Xi still_active method returns whether this finalizer is still waiting to invoke callback
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------    
        class2print = "Finalize4Xi"
        method2print = "still_active"
        

        message2print = "{started" + "; still-active-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
     
        message2print = "{got" + "; key-of-finalize2xi-got-by-still-active-of-Finalize4Xi: " + str(self._key)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                          
        message2print = "{returning" + "; key-of-finalize2xi-in-finalizer4xi-returning-by-still-active-of-Finalize4Xi: " + str(self._key) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                

        
        return self._key in Finalize4Xi._registry
        
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "repr-of-Finalize4Xi" usage = "(self)">                     
    #<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __repr__(self):
        '''
        Finalize4Xi __repr__ method 
        '''
        
        # -----------------------------------------------------------------------------------------
        # Step 1/5 
        # -----------------------------------------------------------------------------------------    
        class2print = "Finalize4Xi"
        method2print = "__repr__"
        
   
        message2print = "{(Step 1/5a) started" + "; repr-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:58:24 01/22/13 Pacific Standard Time
        # 18:40:44 02/22/13 Pacific Standard Time
        
        # -----------------------------------------------------------------------------------------
        # Step 2/5 
        # -----------------------------------------------------------------------------------------            
        try:
            obj = self._weakref()

            message2print = "{(Step 2/5a) got" + "; object-got-by-repr-of-Finalize4Xi: " + str(obj)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <queue4xi.Queue4Xi object at 0x0000000003690550>
            # <bufferwrapper4xi.BufferWrapper4Xi object at 0x0000000002AC2D30>
            
        except (AttributeError, TypeError):
            obj = None

            message2print = "{(Step 2/5b) hit" + "; object-hit-by-repr-of-Finalize4Xi: " + " *** " + str(obj)  + " *** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                                
                                     
        # -----------------------------------------------------------------------------------------
        # Step 3/5 
        # -----------------------------------------------------------------------------------------           
        if obj is None:
           
            message2print = "{(Step 3/5 ) returning" + "; dead-returning-by-repr-of-Finalize4Xi: " + '<finalize4xi: __repr__: Finalize object, dead>' + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                    
            return '<finalize4xi: __repr__: Finalize object, dead>'
        
        # -----------------------------------------------------------------------------------------
        # Step 4/5 
        # -----------------------------------------------------------------------------------------   
        x = '<finalize4xi: __repr__: Finalize object, callback=%s' % \
            getattr(self._callback, '__name__', self._callback)
        
      
        message2print = "{(Step 4/5a) set" + "; x-set-by-repr-of-Finalize4Xi to: " + str(x)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <finalize4xi: __repr__: Finalize object, callback=_finalize_close
        # <finalize4xi: __repr__: Finalize object, callback=free
        
        if self._args:
            x += ', args=' + str(self._args)
       
            message2print = "{(Step 4/5b) set" + "; x-set-by-repr-of-Finalize4Xi to: " + str(x)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <finalize4xi: __repr__: Finalize object, callback=_finalize_close, args=[deque([]), <Condition(<_thread.lock object at 0x0000000003794238>, 0)>];
            # <finalize4xi: __repr__: Finalize object, callback=free, args=((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096),)
            
        if self._kwargs:
            x += ', kwargs=' + str(self._kwargs)
 
            message2print = "{(Step 4/5c) set" + "; x-set-by-repr-of-Finalize4Xi to: " + str(x)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                                
        if self._key[0] is not None:
            x += ', exitprority=' + str(self._key[0])
 
            message2print = "{(Step 4/5d) set" + "; x-set-by-repr-of-Finalize4Xi to: " + str(x)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <finalize4xi: __repr__: Finalize object, callback=_finalize_close, args=[deque([]), <Condition(<_thread.lock object at 0x0000000003794238>, 0)>], exitprority=10                
                                
        # -----------------------------------------------------------------------------------------
        # Step 5/5 
        # -----------------------------------------------------------------------------------------   
        
        message2print = "{(Step 5/5 ) returning" + "; x-returning-by-repr-of-Finalize4Xi: " + str(x + '>') + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
        # <finalize4xi: __repr__: Finalize object, callback=_finalize_close, args=[deque([]), <Condition(<_thread.lock object at 0x0000000003794238>, 0)>], exitprority=10>
        # <finalize4xi: __repr__: Finalize object, callback=free, args=((<arena4xi.Arena4Xi object at 0x00000000035ED080>, 0, 4096),)>
        
        return x + '>'
    #</methodize4xi>
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Finalize4Xi" usage = "senses Finalize4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Finalize4Xi class objects
        '''
        
        class2print = 'Finalize4Xi'
        method2print = 'sense'
        

        message2print = "{started" + "; sense-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Finalize4Xi-sensing-by-sense-of-Finalize4Xi: " + str(Finalize4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Finalize4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
    
        
        message2print = "{sensed" + "; Finalize4Xi-sensed-by-sense-of-Finalize4Xi: " + str(Finalize4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Finalize4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Finalize4X" usage = "reasons Finalize4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Finalize4Xi class objects
        '''
                
        
        class2print = 'Finalize4Xi'
        method2print = 'reason'


        message2print = "{started" + "; reason-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Finalize4Xi-reasoning-by-reason-of-Finalize4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-Finalize4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        message2print = "{reasoned" + "; Finalize4Xi-reasoned-by-reason-of-Finalize4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Finalize4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Finalize4Xi" usage = "aces Finalize4Xi class objects ">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Finalize4Xi class objects 
        '''
                 
        
        
        class2print = 'Finalize4Xi'
        method2print = 'ace'
        

        message2print = "{started" + "; ace-of-Finalize4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
             
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Finalize4Xi-acing-by-ace-of-Finalize4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                     
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Finalize4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; Finalize4Xi-aced-by-ace-of-Finalize4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Finalize4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                
    #</methodize4xi>
    
#</class4xi>

#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of finalize4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

